/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package storage

import (
	"log"
	"os"
	"time"

	"github.com/glebarez/sqlite"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
	"gorm.io/gorm/logger"
)

type Database struct {
	db *gorm.DB
}

func New(dsn string) (db Database, err error) {
	newLogger := logger.New(
		log.New(os.Stderr, "\r\n", log.LstdFlags),
		logger.Config{
			LogLevel: logger.Error, // Log level Info will output everything
		},
	)
	gornDb, err := gorm.Open(sqlite.Open(dsn), &gorm.Config{Logger: newLogger})
	if err != nil {
		return
	}

	db.db = gornDb

	// Migrate the schema
	err = gornDb.AutoMigrate(&ImportTimestamp{})
	return
}

func (db Database) GetImportTimestamp(source string, target string, entity string) (time.Time, error) {
	result := ImportTimestamp{
		SourceBackend: source,
		TargetBackend: target,
		Entity:        entity,
	}
	db.db.Limit(1).Find(&result)
	return result.Timestamp, nil
}

func (db Database) SetImportTimestamp(source string, target string, entity string, timestamp time.Time) error {
	entry := ImportTimestamp{
		SourceBackend: source,
		TargetBackend: target,
		Entity:        entity,
		Timestamp:     timestamp,
	}
	db.db.Clauses(clause.OnConflict{
		DoUpdates: clause.AssignmentColumns([]string{"timestamp", "updated_at"}),
	}).Create(&entry)
	return nil
}
